Event

#[non_exhaustive]
pub enum Event<'a> { Done, FormatTable { fd: BorrowedFd<'a>, size: u32, }, MainDevice { device: Vec<u8>, }, TrancheDone, TrancheTargetDevice { device: Vec<u8>, }, TrancheFormats { indices: Vec<u8>, }, TrancheFlags { flags: TrancheFlags, }, }

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Done

all feedback has been sent

This event is sent after all parameters of a zwp_linux_dmabuf_feedback_v1 object have been sent.

This allows changes to the zwp_linux_dmabuf_feedback_v1 parameters to be seen as atomic, even if they happen via multiple events.



FormatTable

format and modifier table

This event provides a file descriptor which can be memory-mapped to access the format and modifier table.

The table contains a tightly packed array of consecutive format + modifier pairs. Each pair is 16 bytes wide. It contains a format as a 32-bit unsigned integer, followed by 4 bytes of unused padding, and a modifier as a 64-bit unsigned integer. The native endianness is used.

The client must map the file descriptor in read-only private mode.

Compositors are not allowed to mutate the table file contents once this event has been sent. Instead, compositors must create a new, separate table file and re-send feedback parameters. Compositors are allowed to store duplicate format + modifier pairs in the table.

Fields

fd: BorrowedFd<'a>

table file descriptor

size: u32

table size, in bytes



MainDevice

preferred main device

This event advertises the main device that the server prefers to use when direct scan-out to the target device isn’t possible. The advertised main device may be different for each zwp_linux_dmabuf_feedback_v1 object, and may change over time.

There is exactly one main device. The compositor must send at least one preference tranche with tranche_target_device equal to main_device.

Clients need to create buffers that the main device can import and read from, otherwise creating the dmabuf wl_buffer will fail (see the zwp_linux_buffer_params_v1.create and create_immed requests for details). The main device will also likely be kept active by the compositor, so clients can use it instead of waking up another device for power savings.

In general the device is a DRM node. The DRM node type (primary vs. render) is unspecified. Clients must not rely on the compositor sending a particular node type. Clients cannot check two devices for equality by comparing the dev_t value.

If explicit modifiers are not supported and the client performs buffer allocations on a different device than the main device, then the client must force the buffer to have a linear layout.

With version 6 and above, this event is no longer sent. Clients should use a device with the sampling flag in the tranches instead.

Fields

device: Vec<u8>

device dev_t value



TrancheDone

a preference tranche has been sent

This event splits tranche_target_device and tranche_formats events into preference tranches. It is sent after a set of tranche_target_device and tranche_formats events; it represents the end of a tranche. The next tranche will have a lower preference.



TrancheTargetDevice

target device

This event advertises the target device that the server prefers to use for a buffer created given this tranche. The advertised target device may be different for each preference tranche, and may change over time.

There is exactly one target device per tranche.

The target device may be a scan-out device, for example if the compositor prefers to directly scan-out a buffer created given this tranche. The target device may be a rendering device, for example if the compositor prefers to texture from said buffer.

The client can use this hint to allocate the buffer in a way that makes it accessible from the target device, ideally directly. The buffer must still be accessible from a device with the sampling flag, either through direct import or a potentially more expensive fallback path. If the buffer can’t be directly imported for sampling, then clients must be prepared for the compositor changing the tranche priority or making wl_buffer creation fail (see the zwp_linux_buffer_params_v1.create and create_immed requests for details).

If the device is a DRM node, the DRM node type (primary vs. render) is unspecified. Clients must not rely on the compositor sending a particular node type. Clients cannot check two devices for equality by comparing the dev_t value.

This event is tied to a preference tranche, see the tranche_done event.

Fields

device: Vec<u8>

device dev_t value



TrancheFormats

supported buffer format modifiers

This event advertises the format + modifier combinations that the compositor supports.

It carries an array of indices, each referring to a format + modifier pair in the last received format table (see the format_table event). Each index is a 16-bit unsigned integer in native endianness.

For legacy support, DRM_FORMAT_MOD_INVALID is an allowed modifier. It indicates that the server can support the format with an implicit modifier. When a buffer has DRM_FORMAT_MOD_INVALID as its modifier, it is as if no explicit modifier is specified. The effective modifier will be derived from the dmabuf.

A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for a given format supports both explicit modifiers and implicit modifiers.

Compositors must not send duplicate format + modifier pairs within the same tranche or across two different tranches with the same target device and flags.

This event is tied to a preference tranche, see the tranche_done event.

For the definition of the format and modifier codes, see the zwp_linux_buffer_params_v1.create request.

Fields

indices: Vec<u8>

array of 16-bit indexes



TrancheFlags

tranche flags

This event sets tranche-specific flags. This event is tied to a preference tranche, see the tranche_done event. With version 6 and above, the compositor must set at least one flag in each tranche.

Fields

flags: TrancheFlags

tranche flags

Implementations

Source

impl<'a> Event<'a>

pub fn opcode(&self) -> u16

Get the opcode number of this message